require "name_finder/version"
require "name_finder/node_proxy"
require "name_finder/buffer"
require "set"

# Find names from a know list in a text, taking account of names that may
# overlap. For example, Waterloo and Waterloo East are separate stations;
# NameFinder, knowing both, will not give a false match for Waterloo in a text
# that mentions Waterloo East.
#
class NameFinder

  # Initialize a new NameFinder. tree, if supplied, should be the data
  # generated by the export method.
  #
  def initialize(tree={})
    @tree = tree
    @root = NodeProxy.new(tree, delimiter)
  end

  attr_reader :root

  # Add a term to NameFinder's dictionary
  #
  def add(term)
    root.add Buffer.new(normalize(term) + delimiter), term
  end

  # Find the first name from the dictionary in haystack
  #
  def find_in(haystack)
    find(haystack) do |found|
      return found
    end
    nil
  end

  # Find all names from the dictionary in haystack.
  #
  def find_all_in(haystack)
    Set.new.tap { |all|
      find(haystack) do |found|
        all << found
      end
    }.to_a
  end

  # Export the tree of the current dictionary for later re-importing.
  #
  def export
    @tree
  end

private
  def find(haystack)
    remaining = Buffer.new(normalize(haystack) + delimiter)
    while !remaining.at_end?
      found = root.find(remaining)
      if found
        yield found
        remaining = remaining.advance_by(found.length)
      else
        remaining = remaining.advance_past(delimiter)
      end
    end
  end

  def normalize(term)
    term.downcase.gsub(/[^a-z]+/, delimiter)
  end

  def delimiter
    " "
  end
end
